<?php
/**
 * A class for managing the drip downloads feature.
 *
 * @package WooCommerce Subscriptions
 * @since   4.0.0
 */

defined( 'ABSPATH' ) || exit;

class WCS_Drip_Downloads_Manager {

	/**
	 * Initialise the class.
	 *
	 * @since 4.0.0
	 */
	public static function init() {
		add_filter( 'woocommerce_process_product_file_download_paths_grant_access_to_new_file', array( __CLASS__, 'maybe_revoke_immediate_access' ), 10, 4 );
		add_filter( 'woocommerce_subscription_settings', array( __CLASS__, 'add_setting' ) );
	}

	/**
	 * Checks if the drip downloads feature is enabled.
	 *
	 * @since 4.0.0
	 * @return bool Whether download dripping is enabled or not.
	 */
	public static function are_drip_downloads_enabled() {
		return 'yes' === get_option( WC_Subscriptions_Admin::$option_prefix . '_drip_downloadable_content_on_renewal', 'no' );
	}

	/**
	 * Prevent granting download permissions to subscriptions and related-orders when new files are added to a product.
	 *
	 * @since 4.0.0
	 *
	 * @param bool     $grant_access Whether to grant access to the file/download ID.
	 * @param string   $download_id  The ID of the download being added.
	 * @param int      $product_id   The ID of the downloadable product.
	 * @param WC_Order $order        The order/subscription's ID.
	 *
	 * @return bool Whether to grant access to the file/download ID.
	 */
	public static function maybe_revoke_immediate_access( $grant_access, $download_id, $product_id, $order ) {

		if ( self::are_drip_downloads_enabled() && ( wcs_is_subscription( $order->get_id() ) || wcs_order_contains_subscription( $order, 'any' ) ) ) {
			$grant_access = false;
		}

		return $grant_access;
	}

	/**
	 * Adds the Drip Downloadable Content setting.
	 *
	 * @since 4.0.0
	 *
	 * @param array $settings The WC Subscriptions settings array.
	 * @return array Settings.
	 */
	public static function add_setting( $settings ) {
		$setting = array(
			'name'     => __( '滴流传输可下载内容', 'woocommerce-subscriptions' ),
			'desc'     => __( '为订阅产品中的可下载内容启用滴流传输功能。', 'woocommerce-subscriptions' ),
			'id'       => WC_Subscriptions_Admin::$option_prefix . '_drip_downloadable_content_on_renewal',
			'default'  => 'no',
			'type'     => 'checkbox',
			// translators: %s is a line break.
			'desc_tip' => sprintf( __( '启用此功能后，客户仅在处理下一个续订后才能访问添加到产品中的新可下载文件。%s默认情况下，拥有该产品有效订阅的所有客户都可以立即访问添加到该产品的新可下载文件。', 'woocommerce-subscriptions' ), '<br />' ),
		);

		WC_Subscriptions_Admin::insert_setting_after( $settings, WC_Subscriptions_Admin::$option_prefix . '_miscellaneous', $setting );
		return $settings;
	}
}
