<?php
/**
 * WooCommerce Subscriptions User Change Status Handler Class
 *
 * @author      Prospress
 * @since       2.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class WCS_User_Change_Status_Handler {

	public static function init() {
		// Check if a user is requesting to cancel their subscription
		add_action( 'gc_loaded', __CLASS__ . '::maybe_change_users_subscription', 100 );
	}

	/**
	 * Checks if the current request is by a user to change the status of their subscription, and if it is,
	 * validate the request and proceed to change to the subscription.
	 *
	 * @since 2.0
	 */
	public static function maybe_change_users_subscription() {

		if ( isset( $_GET['change_subscription_to'], $_GET['subscription_id'], $_GET['_gcnonce'] ) && ! empty( $_GET['_gcnonce'] ) ) {
			$user_id      = get_current_user_id();
			$subscription = wcs_get_subscription( absint( $_GET['subscription_id'] ) );
			$new_status   = wc_clean( $_GET['change_subscription_to'] );

			if ( self::validate_request( $user_id, $subscription, $new_status, $_GET['_gcnonce'] ) ) {
				self::change_users_subscription( $subscription, $new_status );

				gc_safe_redirect( $subscription->get_view_order_url() );
				exit;
			}
		}
	}

	/**
	 * Change the status of a subscription and show a notice to the user if there was an issue.
	 *
	 * @since 2.0
	 */
	public static function change_users_subscription( $subscription, $new_status ) {
		$subscription = ( ! is_object( $subscription ) ) ? wcs_get_subscription( $subscription ) : $subscription;
		$changed = false;

		do_action( 'woocommerce_before_customer_changed_subscription_to_' . $new_status, $subscription );

		switch ( $new_status ) {
			case 'active':
				if ( ! $subscription->needs_payment() ) {
					$subscription->update_status( $new_status );
					$subscription->add_order_note( _x( '订阅者从其帐户页面重新激活的订阅。', 'order note left on subscription after user action', 'woocommerce-subscriptions' ) );
					wc_add_notice( _x( '您的订阅已重新激活。', 'Notice displayed to user confirming their action.', 'woocommerce-subscriptions' ), 'success' );
					$changed = true;
				} else {
					wc_add_notice( __( '您必须付费续订才能重新激活该订阅。 如需帮助，请与我们联系。', 'woocommerce-subscriptions' ), 'error' );
				}
				break;
			case 'on-hold':
				if ( wcs_can_user_put_subscription_on_hold( $subscription ) ) {
					$subscription->update_status( $new_status );
					$subscription->add_order_note( _x( '订阅者从其帐户页面暂停的订阅。', 'order note left on subscription after user action', 'woocommerce-subscriptions' ) );
					wc_add_notice( _x( '您的订阅已暂停。', 'Notice displayed to user confirming their action.', 'woocommerce-subscriptions' ), 'success' );
					$changed = true;
				} else {
					wc_add_notice( __( '您无法暂停该订阅，因为已达到暂停限制。 如需帮助，请与我们联系。', 'woocommerce-subscriptions' ), 'error' );
				}
				break;
			case 'cancelled':
				$subscription->cancel_order();
				$subscription->add_order_note( _x( '订阅者从其帐户页面取消的订阅。', 'order note left on subscription after user action', 'woocommerce-subscriptions' ) );
				wc_add_notice( _x( '您的订阅已被取消。', 'Notice displayed to user confirming their action.', 'woocommerce-subscriptions' ), 'success' );
				$changed = true;
				break;
		}

		if ( $changed ) {
			do_action( 'woocommerce_customer_changed_subscription_to_' . $new_status, $subscription );
		}
	}

	/**
	 * Validates a user change status change request.
	 *
	 * @since 2.0.0
	 *
	 * @param int             $user_id The ID of the user performing the request.
	 * @param WC_Subscription $subscription The Subscription to update.
	 * @param string          $new_status The new subscription status to validate.
	 * @param string|null     $gcnonce Optional. The nonce to validate the request or null if there's no nonce to validate.
	 *
	 * @return bool Whether the status change request is valid.
	 */
	public static function validate_request( $user_id, $subscription, $new_status, $gcnonce = null ) {
		$subscription = ( ! is_object( $subscription ) ) ? wcs_get_subscription( $subscription ) : $subscription;

		if ( ! wcs_is_subscription( $subscription ) ) {
			wc_add_notice( __( '该订阅不存在。 如需帮助，请与我们联系。', 'woocommerce-subscriptions' ), 'error' );
			return false;

		} elseif ( isset( $gcnonce ) && gc_verify_nonce( $gcnonce, $subscription->get_id() . $subscription->get_status() ) === false ) {
			wc_add_notice( __( '安全错误。 如需帮助，请与我们联系。', 'woocommerce-subscriptions' ), 'error' );
			return false;

		} elseif ( ! user_can( $user_id, 'edit_shop_subscription_status', $subscription->get_id() ) ) {
			wc_add_notice( __( '那似乎不是您的订阅之一。', 'woocommerce-subscriptions' ), 'error' );
			return false;

		} elseif ( ! $subscription->can_be_updated_to( $new_status ) ) {
			// translators: placeholder is subscription's new status, translated
			wc_add_notice( sprintf( __( '该订阅不能更改为%s。 如需帮助，请与我们联系。', 'woocommerce-subscriptions' ), wcs_get_subscription_status_name( $new_status ) ), 'error' );
			return false;
		}

		return true;
	}
}
